/*
 * uart.c
 *
 *  Created on: Dec 15, 2014
 *      Author: shino-chan
 */
//USART2
//PA2 => TX
//PA3 => RX
//Alternate Function 7
#include <uart/uart.h>
#include <stm32f4xx.h>
#include <clock-enable-manager.h>
#include <stm32-gpio-macros.h>
#include <uart/dma-ring-buffer.h>

static struct dma_ring_buffer_to_mem ring_buff_rx;
static char uart_rx_buffer[64];


#ifdef DEBUGBUILD
static inline void uart_gpio_config()
{
	rcc_manager_enable_clock(&RCC->AHB1ENR, BITMASK_TO_BITNO(UART_PORT_RCC_MASK));
	UART_PORT->MODER &=  MODER_DELETE(UART_TX_PIN) & MODER_DELETE(UART_RX_PIN);
	UART_PORT->MODER |= ALTFUNC(UART_RX_PIN) | ALTFUNC(UART_TX_PIN);
	SETAF(UART_PORT, UART_RX_PIN, UART_RX_PIN_ALTFUNC);
	SETAF(UART_PORT, UART_TX_PIN, UART_TX_PIN_ALTFUNC);
}
#endif

void uart_init_with_dma()
{
	rcc_manager_enable_clock(&RCC->APB2ENR, BITMASK_TO_BITNO(UART_RCC_MASK));
#ifdef DEBUGBUILD
	uart_gpio_config();
#endif
	UART_PERIPH->BRR = UART_BRR_REG_VALUE;
	UART_PERIPH->CR3 = USART_CR3_DMAR | USART_CR3_DMAT;
	UART_PERIPH->CR1 = USART_CR1_TE | USART_CR1_RE | USART_CR1_UE;

	dma_ring_buffer_periph_to_mem_initialize(&ring_buff_rx, 2, UART_RECEIVE_DMA_STREAM, sizeof(uart_rx_buffer), 1U,
						 uart_rx_buffer, (char *)&UART_PERIPH->DR, 4);
}

void uart_disable()
{
	UART_PERIPH->CR1 = 0;
	UART_PERIPH->CR2 = 0;
	UART_PERIPH->CR3 = 0;
	dma_ring_buffer_periph_to_mem_stop(&ring_buff_rx);

	rcc_manager_disable_clock(&RCC->AHB1ENR, BITMASK_TO_BITNO(UART_PORT_RCC_MASK));
	rcc_manager_disable_clock(&RCC->APB2ENR, BITMASK_TO_BITNO(UART_RCC_MASK));
}

void uart_send_char(char c)
{
	while(!(UART_PERIPH->SR & USART_SR_TXE));
	UART_PERIPH->DR = c;
}

void uart_send_array(const char *data, uint32_t len)
{
	uint32_t i;

	for (i = 0; i < len; i++)
		uart_send_char(data[i]);
}

void uart_send_string(char *string)
{
	int i;

	for (i = 0; string[i] != '\0'; i++)
		uart_send_char(string[i]);
}

void uart_send_array_with_dma(char *data, uint32_t len);

void uart_send_string_with_dma(char *string);

int uart_receive_data_with_dma(const char **data, size_t *len)
{
	return dma_ring_buffer_periph_to_mem_get_data(&ring_buff_rx, (const void **)data, len);
}
